# This is temporary as we should use standard DRF renderers
import json

from rest_framework import exceptions


class BaseRenderer:
    pass


class SurveyKitRenderer(BaseRenderer):
    FORMATS = {
        # A dictionary to rewrite internal type and format into survey kit's fmt
        "text": {
            "defaults": {
                "type": "text",
            },
        },
        "integer": {
            "defaults": {
                "type": "integer",
            },
            "rewrites": {
                "maximumValue": "max",
                "minimumValue": "min",
            },
        },
        "date": {
            "defaults": {
                "type": "date",
            },
            "rewrites": {"maxDate": "max", "minDate": "min"},
        },
        "single": {
            "defaults": {
                "type": "single",
                "otherField": False,
            },
            "rewrites": {
                "textChoices": "choices",
            },
        },
        "multi": {
            "defaults": {
                "type": "multiple",
                "otherField": False,
            },
            "rewrites": {
                "textChoices": "choices",
            },
        },
    }

    @classmethod
    def parse_format(self, fmt: str) -> dict:
        if fmt.startswith("{"):
            return json.loads(fmt)
        if ".." in fmt:
            min_, max_ = fmt.split("..")
            return {
                "min": min_,
                "max": max_,
            }
        if "|" in fmt:
            return {"choices": [{"value": v, "text": v} for v in fmt.split("|")]}
        return {}

    @classmethod
    def generate_task_from_survey(cls, survey):
        steps = []
        for question in survey.question_set.all():
            step = {
                "type": "question",
                "title": question.question,
                "text": question.text,
                "stepIdentifier": {"id": str(question.id)},
            }

            if question.type == "info":
                step_type, button_text = question.format.split("|")
                step["type"] = step_type
                step["buttonText"] = button_text
            else:
                try:
                    f = SurveyKitRenderer.FORMATS[question.type]
                except KeyError:
                    raise exceptions.ValidationError(
                        f"Unrecognized question type '{question.type}'"
                    )
                question_format = cls.parse_format(question.format)
                step["answerFormat"] = {
                    **f.get("defaults", {}),
                    **{
                        k: question_format.get(v)
                        for k, v in f.get("rewrites", {}).items()
                        if v in question_format
                    },
                }

            steps.append(step)

        return {
            "id": str(survey.id),
            "type": "navigable",
            "rules": [],
            "steps": steps,
        }



# Generated by Django 4.2.6 on 2023-11-21 12:08
from datetime import timedelta

from django.db import migrations


def init_data(apps, schema_editor):
    UserSurveyNotification = apps.get_model("df_survey", "UserSurveyNotification")
    UserSurveyNotification.objects.create(
        channel="push",
        template_prefix="df_notifications/usersurveynotification/",
    )
    UserSurveyNotification.objects.create(
        channel="email",
        template_prefix="df_notifications/usersurveynotification/",
    )
    UserSurveysReminder = apps.get_model("df_survey", "UserSurveysReminder")
    UserSurveysReminder.objects.create(
        channel="push",
        template_prefix="df_notifications/usersurveysreminder/",
        delay=timedelta(hours=1),
    )
    UserSurveysReminder.objects.create(
        channel="email",
        template_prefix="df_notifications/usersurveysreminder/",
        delay=timedelta(days=1),
    )


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("df_survey", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(init_data, noop),
    ]



from rest_framework import mixins
from rest_framework.exceptions import ValidationError
from rest_framework.viewsets import GenericViewSet

from ..models import UserSurvey
from .serializers import UserSurveyDetailsSerializer, UserSurveySerializer


class UserSurveyViewSet(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    mixins.RetrieveModelMixin,
    GenericViewSet,
):
    serializer_class = UserSurveyDetailsSerializer
    queryset = UserSurvey.objects.all()

    def get_queryset(self):
        return self.queryset.filter(
            user=self.request.user,
            survey__task__isnull=False,
        )

    def get_serializer_class(self):
        if self.action == "list":
            return UserSurveySerializer
        return self.serializer_class

    def perform_update(self, serializer):
        if serializer.instance.result:
            raise ValidationError("Your submission has already been recorded.")
        return super().perform_update(serializer)

    def get_object(self):
        pk = self.kwargs.get("pk", "")
        if pk.startswith("@"):
            survey_pk = pk[1:]
            return UserSurvey.objects.get_or_create(
                user=self.request.user,
                survey_id=survey_pk,
            )[0]
        return super().get_object()



# Generated by Django 4.2.6 on 2023-11-21 08:59

import datetime
import df_notifications.fields
import df_notifications.models
import df_survey.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import hashid_field.field
import model_utils.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("df_notifications", "0008_alter_custompushmessage_action_url_and_more"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("slug", models.CharField(max_length=128)),
            ],
            options={
                "verbose_name_plural": "Survey categories",
            },
        ),
        migrations.CreateModel(
            name="Question",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("question", models.CharField(max_length=255)),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("text", "Text"),
                            ("integer", "Integer"),
                            ("date", "Date"),
                            ("single", "Single choice"),
                            ("multi", "Multiple choice"),
                        ],
                        max_length=255,
                    ),
                ),
                ("format", models.TextField(blank=True, default="")),
            ],
        ),
        migrations.CreateModel(
            name="Survey",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=128)),
                ("description", models.TextField(blank=True, null=True)),
                (
                    "sequence",
                    models.PositiveSmallIntegerField(
                        default=1000, help_text="Display sequence, lower means first"
                    ),
                ),
                (
                    "task",
                    models.JSONField(
                        blank=True,
                        null=True,
                        validators=[df_survey.models.validate_task_json],
                    ),
                ),
                (
                    "category",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="df_survey.category",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="UserSurveysReminder",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "channel",
                    df_notifications.fields.NoMigrationsChoicesField(max_length=255),
                ),
                ("template_prefix", models.CharField(max_length=255)),
                ("context", models.JSONField(blank=True, default=dict)),
                (
                    "delay",
                    models.DurationField(
                        default=datetime.timedelta(0),
                        help_text="Send the reminder after this period of time",
                    ),
                ),
                (
                    "cooldown",
                    models.DurationField(
                        default=datetime.timedelta(seconds=3600),
                        help_text="Wait so much time before reminding again",
                    ),
                ),
                (
                    "repeat",
                    models.SmallIntegerField(
                        default=1, help_text="Repeat the reminder this many times"
                    ),
                ),
                (
                    "action",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="Python code to execute. You can use `instance` variable to access current model",
                    ),
                ),
                (
                    "history",
                    models.ManyToManyField(
                        blank=True,
                        editable=False,
                        to="df_notifications.notificationhistory",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(df_notifications.models.GenericBase, models.Model),
        ),
        migrations.CreateModel(
            name="UserSurveyNotification",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "channel",
                    df_notifications.fields.NoMigrationsChoicesField(max_length=255),
                ),
                ("template_prefix", models.CharField(max_length=255)),
                ("context", models.JSONField(blank=True, default=dict)),
                (
                    "history",
                    models.ManyToManyField(
                        blank=True,
                        editable=False,
                        to="df_notifications.notificationhistory",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(df_notifications.models.GenericBase, models.Model),
        ),
        migrations.CreateModel(
            name="UserSurvey",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="created",
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="modified",
                    ),
                ),
                ("result", models.JSONField(blank=True, null=True)),
                (
                    "survey",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="df_survey.survey",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-modified"],
            },
        ),
        migrations.CreateModel(
            name="SurveyQuestion",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "sequence",
                    models.PositiveSmallIntegerField(
                        default=1000, help_text="Display sequence, lower means first"
                    ),
                ),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="df_survey.question",
                    ),
                ),
                (
                    "survey",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="df_survey.survey",
                    ),
                ),
            ],
            options={
                "ordering": ("sequence",),
                "unique_together": {("survey", "question")},
            },
        ),
        migrations.AddField(
            model_name="survey",
            name="questions",
            field=models.ManyToManyField(
                through="df_survey.SurveyQuestion", to="df_survey.question"
            ),
        ),
        migrations.CreateModel(
            name="Response",
            fields=[
                (
                    "id",
                    hashid_field.field.BigHashidAutoField(
                        alphabet="ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
                        auto_created=True,
                        min_length=13,
                        prefix="",
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("response", models.TextField()),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="df_survey.question",
                    ),
                ),
                (
                    "usersurvey",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="df_survey.usersurvey",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 4.2.6 on 2023-11-23 11:10

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("df_survey", "0004_alter_usersurvey_unique_together"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="response",
            unique_together={("usersurvey", "question")},
        ),
    ]



import json

from django.template import Context, Template
from drf_spectacular.utils import extend_schema_field
from hashid_field.rest import HashidSerializerCharField
from rest_framework import serializers

from ..models import UserSurvey


class UserSurveySerializer(serializers.ModelSerializer):
    id = HashidSerializerCharField(read_only=True)
    title = serializers.CharField(source="survey.title", read_only=True)
    description = serializers.CharField(source="survey.description", read_only=True)
    category = serializers.CharField(source="survey.category.slug", read_only=True)
    completed = serializers.DateTimeField(read_only=True)

    def to_representation(self, instance):
        attrs = super().to_representation(instance)
        if instance.result:
            attrs["completed"] = instance.modified
        return attrs

    class Meta:
        model = UserSurvey
        read_only_fields = ("created", "modified")
        fields = read_only_fields + (
            "id",
            "title",
            "category",
            "description",
            "completed",
        )


class UserSurveyDetailsSerializer(UserSurveySerializer):
    task = serializers.SerializerMethodField("get_task")
    result = serializers.JSONField(required=False)

    def validate(self, attrs):
        attrs = super().validate(attrs)
        attrs["user"] = self.context["request"].user
        return attrs

    class Meta(UserSurveySerializer.Meta):
        fields = (*UserSurveySerializer.Meta.fields, "task", "result")

    @extend_schema_field(serializers.JSONField)
    def get_task(self, obj: UserSurvey):
        template = Template(json.dumps(obj.survey.task))
        task_str = template.render(Context({"user": obj.user}))
        task = json.loads(task_str)

        results = {res.step_id: res for res in obj.pretty_results()}

        for step in task["steps"]:
            result = results.get(step["stepIdentifier"]["id"])
            if not result:
                continue

            if "answerFormat" not in step:
                step["answerFormat"] = {}

            step["answerFormat"]["defaultValue"] = result.answer_full
            step["answerFormat"]["defaultSelection"] = result.answer_full

        return task
